;;; -*- mode: lisp-interaction -*-

;; Summary: use `compute-motion' to measure the buffer position of the visual
;; coordinate '(0 . 30) in a new buffer with some content. Here, column 30 is
;; at (after) buffer position 31.
;;
;; `compute-motion' calculates this correctly unless it scans over a
;; 'before-string or 'after-string overlay of the style shown in
;; `testcase-apply-overlay' in which case it returns buffer position 28.

(defun testcase-recreate-buffer (testcase-buffer)
  (when (get-buffer testcase-buffer)
    (kill-buffer testcase-buffer))
  (with-current-buffer (get-buffer-create testcase-buffer)
    (insert "     5    10   15   20   25   30   35   40   45   50\n")
    (goto-char (point-min))))

(defun testcase-measure-at-col-30 (testcase-buffer)
  (let ((prev-buffer (current-buffer)))
    (switch-to-buffer testcase-buffer)
    (goto-char (point-min))
    (move-to-column 30)
    (redisplay)  ; hack: make posn-at-point return non-nil
    (message "- `point' at col 30: <%s>" (point))
    (message "- buffer content on and after col 30: <%s>"
             (buffer-substring (point) (line-end-position)))
    (message "- posn coord at col 30: <%s>" (posn-col-row (posn-at-point)))
    (pcase-let ((`(,pos ,col ,row _ _)
                 (compute-motion (point-min)
                                 '(0 . 0)
                                 (point-max)
                                 (posn-col-row (posn-at-point))
                                 nil nil nil)))
      (message "- buffer pos of col 30 according to `compute-motion': <%s> %s"
               pos
               (if (eq pos (point))
                   "    [ok]"
                 "    <<< INCONSISTENT"))
      (message "- coord of col 30 according to `compute-motion': <%s>"
               (cons col row)))
    (switch-to-buffer prev-buffer)))

(defun testcase-apply-overlay (buf overlay-pos prop-sym)
  (with-current-buffer buf
    (let* ((ol (make-overlay overlay-pos overlay-pos
                             (current-buffer) nil nil)))
      (overlay-put ol
                   prop-sym
                   (propertize "AAA" 'display ""))
      ol)))

(let ((testcase-buffer "*compute-motion bug*"))
  (message "\n=======================================\n")
  (let ((prev-buffer (current-buffer)))
    (message "-------------------------------------")
    (message "Without overlay:")
    (testcase-recreate-buffer testcase-buffer)
    (testcase-measure-at-col-30 testcase-buffer)

    (message "-------------------------------------")
    (message "With 'before-string overlay:")
    (testcase-recreate-buffer testcase-buffer)
    (testcase-apply-overlay testcase-buffer 20 'before-string)
    (testcase-measure-at-col-30 testcase-buffer)

    (message "-------------------------------------")
    (message "With 'after-string overlay:")
    (testcase-recreate-buffer testcase-buffer)
    (testcase-apply-overlay testcase-buffer 20 'after-string)
    (testcase-measure-at-col-30 testcase-buffer)

    (switch-to-buffer prev-buffer))
  nil)

